@@ -155,11 +155,12 @@ def calculate_max_checkpoint_remap(size_bytes: int) -> int:
 
 def update_ini_memory_settings(ini_path: str, data_dir_path: str, number_of_buffers: int = None, max_dirty_buffers: int = None, dirs_allowed: str = None):
     """
-    Updates memory-related settings in the virtuoso.ini file:
+    Updates settings in the virtuoso.ini file:
     - MaxCheckpointRemap: based on the actual size of the data directory
     - NumberOfBuffers: if provided
     - MaxDirtyBuffers: if provided
     - DirsAllowed: if provided
+    - [Client] SQL_QUERY_TIMEOUT and SQL_TXN_TIMEOUT set to 0
 
     Args:
         ini_path: The full path to the virtuoso.ini file.
@@ -200,6 +201,23 @@ def update_ini_memory_settings(ini_path: str, data_dir_path: str, number_of_buff
                 print(f"Info: Updating [Parameters] NumberOfBuffers from '{current_number_of_buffers}' to '{number_of_buffers_str}' in '{ini_path}'.")
                 made_changes = True
 
+        # Ensure [Client] section has SQL timeouts set to 0
+        if not config.has_section('Client'):
+            config.add_section('Client')
+            print(f"Info: Added [Client] section to '{ini_path}'.")
+
+        current_sql_query_timeout = config.get('Client', 'SQL_QUERY_TIMEOUT', fallback=None)
+        if current_sql_query_timeout != '0':
+            config.set('Client', 'SQL_QUERY_TIMEOUT', '0')
+            print(f"Info: Setting [Client] SQL_QUERY_TIMEOUT to '0' in '{ini_path}'.")
+            made_changes = True
+
+        current_sql_txn_timeout = config.get('Client', 'SQL_TXN_TIMEOUT', fallback=None)
+        if current_sql_txn_timeout != '0':
+            config.set('Client', 'SQL_TXN_TIMEOUT', '0')
+            print(f"Info: Setting [Client] SQL_TXN_TIMEOUT to '0' in '{ini_path}'.")
+            made_changes = True
+
         # Update MaxDirtyBuffers if provided
         if max_dirty_buffers is not None:
             current_max_dirty_buffers = config.get('Parameters', 'MaxDirtyBuffers', fallback=None)
@@ -400,6 +418,18 @@ def parse_arguments() -> argparse.Namespace:
              "based on this estimate rather than measuring existing data."
     )
     
+    parser.add_argument(
+        "--virtuoso-version",
+        default=None,
+        help="Virtuoso Docker image version/tag to use (e.g., 'latest', '7.2.11', '7.2.12'). If not specified, uses the default pinned version."
+    )
+    
+    parser.add_argument(
+        "--virtuoso-sha",
+        default=None,
+        help="Virtuoso Docker image SHA256 digest to use (e.g., 'sha256:e07868a3db9090400332eaa8ee694b8cf9bf7eebc26db6bbdc3bb92fd30ed010'). Takes precedence over --virtuoso-version."
+    )
+    
     args_temp, _ = parser.parse_known_args()
     
     optimal_number_of_buffers, optimal_max_dirty_buffers = get_optimal_buffer_values(args_temp.memory)
@@ -481,6 +511,27 @@ def remove_container(container_name: str) -> bool:
         return False
 
 
+def get_docker_image(version: str, sha: str) -> str:
+    """
+    Get the appropriate Docker image based on version or SHA parameter.
+    
+    Args:
+        version: Version string (e.g., 'latest', '7.2.11', '7.2.12') or None for default
+        sha: SHA256 digest string or None
+        
+    Returns:
+        str: Full Docker image reference
+    """
+    if sha is not None:
+        return f"openlink/virtuoso-opensource-7@{sha}"
+    elif version is None:
+        return DEFAULT_IMAGE
+    elif version == "latest":
+        return "openlink/virtuoso-opensource-7:latest"
+    else:
+        return f"openlink/virtuoso-opensource-7:{version}"
+
+
 def build_docker_run_command(args: argparse.Namespace) -> Tuple[List[str], List[str]]:
     """
     Build the Docker run command based on provided arguments.
@@ -548,7 +599,10 @@ def build_docker_run_command(args: argparse.Namespace) -> Tuple[List[str], List[
         "VIRT_Parameters_MaxDirtyBuffers": str(args.max_dirty_buffers),
         "VIRT_Parameters_NumberOfBuffers": str(args.number_of_buffers),
         "VIRT_Parameters_DirsAllowed": ",".join(paths_to_allow_in_container),
-        "VIRT_SPARQL_DefaultQuery": "SELECT (COUNT(*) AS ?quadCount) WHERE { GRAPH ?g { ?s ?p ?o } }"
+        "VIRT_SPARQL_DefaultQuery": "SELECT (COUNT(*) AS ?quadCount) WHERE { GRAPH ?g { ?s ?p ?o } }",
+        # Enforce client-side timeouts to 0
+        "VIRT_Client_SQL_QUERY_TIMEOUT": "0",
+        "VIRT_Client_SQL_TXN_TIMEOUT": "0",
     }
     
     # Set MaxCheckpointRemap environment variables if estimated size is provided
@@ -571,7 +625,8 @@ def build_docker_run_command(args: argparse.Namespace) -> Tuple[List[str], List[
         cmd.insert(2, "--rm") # Insert after "docker run"
     
     # Append image name
-    cmd.append(DEFAULT_IMAGE)
+    docker_image = get_docker_image(args.virtuoso_version, args.virtuoso_sha)
+    cmd.append(docker_image)
     
     return cmd, paths_to_allow_in_container
 
@@ -830,4 +885,4 @@ Virtuoso launched successfully!
 
 
 if __name__ == "__main__":
-    sys.exit(main()) 
\ No newline at end of file
+    sys.exit(main()) 
